@@ -15,8 +15,9 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
-
-from cStringIO import StringIO
+from paramiko.common import linefeed_byte_value, crlf, cr_byte, linefeed_byte, \
+    cr_byte_value
+from paramiko.py3compat import BytesIO, PY2, u, b, bytes_types
 
 
 class BufferedFile (object):
@@ -43,8 +44,8 @@ class BufferedFile (object):
         self.newlines = None
         self._flags = 0
         self._bufsize = self._DEFAULT_BUFSIZE
-        self._wbuffer = StringIO()
-        self._rbuffer = ''
+        self._wbuffer = BytesIO()
+        self._rbuffer = bytes()
         self._at_trailing_cr = False
         self._closed = False
         # pos - position within the file, according to the user
@@ -82,23 +83,40 @@ class BufferedFile (object):
         buffering is not turned on.
         """
         self._write_all(self._wbuffer.getvalue())
-        self._wbuffer = StringIO()
+        self._wbuffer = BytesIO()
         return
 
-    def next(self):
-        """
-        Returns the next line from the input, or raises
-        `~exceptions.StopIteration` when EOF is hit.  Unlike Python file
-        objects, it's okay to mix calls to `next` and `readline`.
+    if PY2:
+        def next(self):
+            """
+            Returns the next line from the input, or raises
+            `~exceptions.StopIteration` when EOF is hit.  Unlike Python file
+            objects, it's okay to mix calls to `next` and `readline`.
 
-        :raises StopIteration: when the end of the file is reached.
+            :raises StopIteration: when the end of the file is reached.
 
-        :return: a line (`str`) read from the file.
-        """
-        line = self.readline()
-        if not line:
-            raise StopIteration
-        return line
+            :return: a line (`str`) read from the file.
+            """
+            line = self.readline()
+            if not line:
+                raise StopIteration
+            return line
+    else:
+        def __next__(self):
+            """
+            Returns the next line from the input, or raises L{StopIteration} when
+            EOF is hit.  Unlike python file objects, it's okay to mix calls to
+            C{next} and L{readline}.
+
+            @raise StopIteration: when the end of the file is reached.
+
+            @return: a line read from the file.
+            @rtype: str
+            """
+            line = self.readline()
+            if not line:
+                raise StopIteration
+            return line
 
     def read(self, size=None):
         """
@@ -118,7 +136,7 @@ class BufferedFile (object):
         if (size is None) or (size < 0):
             # go for broke
             result = self._rbuffer
-            self._rbuffer = ''
+            self._rbuffer = bytes()
             self._pos += len(result)
             while True:
                 try:
@@ -130,12 +148,12 @@ class BufferedFile (object):
                 result += new_data
                 self._realpos += len(new_data)
                 self._pos += len(new_data)
-            return result
+            return result if self._flags & self.FLAG_BINARY else u(result)
         if size <= len(self._rbuffer):
             result = self._rbuffer[:size]
             self._rbuffer = self._rbuffer[size:]
             self._pos += len(result)
-            return result
+            return result if self._flags & self.FLAG_BINARY else u(result)
         while len(self._rbuffer) < size:
             read_size = size - len(self._rbuffer)
             if self._flags & self.FLAG_BUFFERED:
@@ -151,7 +169,7 @@ class BufferedFile (object):
         result = self._rbuffer[:size]
         self._rbuffer = self._rbuffer[size:]
         self._pos += len(result)
-        return result
+        return result if self._flags & self.FLAG_BINARY else u(result)
 
     def readline(self, size=None):
         """
@@ -181,11 +199,11 @@ class BufferedFile (object):
             if self._at_trailing_cr and (self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (len(line) > 0):
                 # edge case: the newline may be '\r\n' and we may have read
                 # only the first '\r' last time.
-                if line[0] == '\n':
+                if line[0] == linefeed_byte_value:
                     line = line[1:]
-                    self._record_newline('\r\n')
+                    self._record_newline(crlf)
                 else:
-                    self._record_newline('\r')
+                    self._record_newline(cr_byte)
                 self._at_trailing_cr = False
             # check size before looking for a linefeed, in case we already have
             # enough.
@@ -195,42 +213,42 @@ class BufferedFile (object):
                     self._rbuffer = line[size:]
                     line = line[:size]
                     self._pos += len(line)
-                    return line
+                    return line if self._flags & self.FLAG_BINARY else u(line)
                 n = size - len(line)
             else:
                 n = self._bufsize
-            if ('\n' in line) or ((self._flags & self.FLAG_UNIVERSAL_NEWLINE) and ('\r' in line)):
+            if (linefeed_byte in line) or ((self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (cr_byte in line)):
                 break
             try:
                 new_data = self._read(n)
             except EOFError:
                 new_data = None
             if (new_data is None) or (len(new_data) == 0):
-                self._rbuffer = ''
+                self._rbuffer = bytes()
                 self._pos += len(line)
-                return line
+                return line if self._flags & self.FLAG_BINARY else u(line)
             line += new_data
             self._realpos += len(new_data)
         # find the newline
-        pos = line.find('\n')
+        pos = line.find(linefeed_byte)
         if self._flags & self.FLAG_UNIVERSAL_NEWLINE:
-            rpos = line.find('\r')
-            if (rpos >= 0) and ((rpos < pos) or (pos < 0)):
+            rpos = line.find(cr_byte)
+            if (rpos >= 0) and (rpos < pos or pos < 0):
                 pos = rpos
         xpos = pos + 1
-        if (line[pos] == '\r') and (xpos < len(line)) and (line[xpos] == '\n'):
+        if (line[pos] == cr_byte_value) and (xpos < len(line)) and (line[xpos] == linefeed_byte_value):
             xpos += 1
         self._rbuffer = line[xpos:]
         lf = line[pos:xpos]
-        line = line[:pos] + '\n'
-        if (len(self._rbuffer) == 0) and (lf == '\r'):
+        line = line[:pos] + linefeed_byte
+        if (len(self._rbuffer) == 0) and (lf == cr_byte):
             # we could read the line up to a '\r' and there could still be a
             # '\n' following that we read next time.  note that and eat it.
             self._at_trailing_cr = True
         else:
             self._record_newline(lf)
         self._pos += len(line)
-        return line
+        return line if self._flags & self.FLAG_BINARY else u(line)
 
     def readlines(self, sizehint=None):
         """
@@ -243,14 +261,14 @@ class BufferedFile (object):
         :return: `list` of lines read from the file.
         """
         lines = []
-        bytes = 0
+        byte_count = 0
         while True:
             line = self.readline()
             if len(line) == 0:
                 break
             lines.append(line)
-            bytes += len(line)
-            if (sizehint is not None) and (bytes >= sizehint):
+            byte_count += len(line)
+            if (sizehint is not None) and (byte_count >= sizehint):
                 break
         return lines
 
@@ -292,6 +310,7 @@ class BufferedFile (object):
 
         :param str data: data to write
         """
+        data = b(data)
         if self._closed:
             raise IOError('File is closed')
         if not (self._flags & self.FLAG_WRITE):
@@ -302,12 +321,12 @@ class BufferedFile (object):
         self._wbuffer.write(data)
         if self._flags & self.FLAG_LINE_BUFFERED:
             # only scan the new data for linefeed, to avoid wasting time.
-            last_newline_pos = data.rfind('\n')
+            last_newline_pos = data.rfind(linefeed_byte)
             if last_newline_pos >= 0:
                 wbuf = self._wbuffer.getvalue()
                 last_newline_pos += len(wbuf) - len(data)
                 self._write_all(wbuf[:last_newline_pos + 1])
-                self._wbuffer = StringIO()
+                self._wbuffer = BytesIO()
                 self._wbuffer.write(wbuf[last_newline_pos + 1:])
             return
         # even if we're line buffering, if the buffer has grown past the
@@ -340,10 +359,8 @@ class BufferedFile (object):
     def closed(self):
         return self._closed
 
-
     ###  overrides...
 
-
     def _read(self, size):
         """
         (subclass override)
@@ -370,10 +387,8 @@ class BufferedFile (object):
         """
         return 0
 
-
     ###  internals...
 
-
     def _set_mode(self, mode='r', bufsize=-1):
         """
         Subclasses call this method to initialize the BufferedFile.
@@ -401,13 +416,13 @@ class BufferedFile (object):
             self._flags |= self.FLAG_READ
         if ('w' in mode) or ('+' in mode):
             self._flags |= self.FLAG_WRITE
-        if ('a' in mode):
+        if 'a' in mode:
             self._flags |= self.FLAG_WRITE | self.FLAG_APPEND
             self._size = self._get_size()
             self._pos = self._realpos = self._size
-        if ('b' in mode):
+        if 'b' in mode:
             self._flags |= self.FLAG_BINARY
-        if ('U' in mode):
+        if 'U' in mode:
             self._flags |= self.FLAG_UNIVERSAL_NEWLINE
             # built-in file objects have this attribute to store which kinds of
             # line terminations they've seen:
@@ -436,7 +451,7 @@ class BufferedFile (object):
             return
         if self.newlines is None:
             self.newlines = newline
-        elif isinstance(self.newlines, basestring) and (self.newlines != newline):
+        elif self.newlines != newline and isinstance(self.newlines, bytes_types):
             self.newlines = (self.newlines, newline)
         elif newline not in self.newlines:
             self.newlines += (newline,)
