@@ -21,8 +21,9 @@
 """
 
 import threading
-from paramiko.common import *
 from paramiko import util
+from paramiko.common import DEBUG, ERROR, OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED, AUTH_FAILED
+from paramiko.py3compat import string_types
 
 
 class ServerInterface (object):
@@ -291,10 +292,8 @@ class ServerInterface (object):
         """
         return False
 
-
     ###  Channel requests
 
-
     def check_channel_pty_request(self, channel, term, width, height, pixelwidth, pixelheight,
                                   modes):
         """
@@ -514,7 +513,7 @@ class InteractiveQuery (object):
         self.instructions = instructions
         self.prompts = []
         for x in prompts:
-            if (type(x) is str) or (type(x) is unicode):
+            if isinstance(x, string_types):
                 self.add_prompt(x)
             else:
                 self.add_prompt(x[0], x[1])
@@ -576,7 +575,7 @@ class SubsystemHandler (threading.Thread):
         try:
             self.__transport._log(DEBUG, 'Starting handler for subsystem %s' % self.__name)
             self.start_subsystem(self.__name, self.__transport, self.__channel)
-        except Exception, e:
+        except Exception as e:
             self.__transport._log(ERROR, 'Exception in subsystem handler for "%s": %s' %
                                   (self.__name, str(e)))
             self.__transport._log(ERROR, util.tb_strings())
