@@ -23,19 +23,15 @@ a real actual sftp server is contacted, and a new folder is created there to
 do test file operations in (so no existing files will be harmed).
 """
 
-import logging
 import os
 import random
 import struct
 import sys
-import threading
 import time
 import unittest
 
-import paramiko
-from stub_sftp import StubServer, StubSFTPServer
-from loop import LoopSocket
-from test_sftp import get_sftp
+from paramiko.common import o660
+from tests.test_sftp import get_sftp
 
 FOLDER = os.environ.get('TEST_FOLDER', 'temp-testing000')
 
@@ -45,7 +41,7 @@ class BigSFTPTest (unittest.TestCase):
     def setUp(self):
         global FOLDER
         sftp = get_sftp()
-        for i in xrange(1000):
+        for i in range(1000):
             FOLDER = FOLDER[:-3] + '%03d' % i
             try:
                 sftp.mkdir(FOLDER)
@@ -65,19 +61,17 @@ class BigSFTPTest (unittest.TestCase):
         numfiles = 100
         try:
             for i in range(numfiles):
-                f = sftp.open('%s/file%d.txt' % (FOLDER, i), 'w', 1)
-                f.write('this is file #%d.\n' % i)
-                f.close()
-                sftp.chmod('%s/file%d.txt' % (FOLDER, i), 0660)
+                with sftp.open('%s/file%d.txt' % (FOLDER, i), 'w', 1) as f:
+                    f.write('this is file #%d.\n' % i)
+                sftp.chmod('%s/file%d.txt' % (FOLDER, i), o660)
 
             # now make sure every file is there, by creating a list of filenmes
             # and reading them in random order.
-            numlist = range(numfiles)
+            numlist = list(range(numfiles))
             while len(numlist) > 0:
                 r = numlist[random.randint(0, len(numlist) - 1)]
-                f = sftp.open('%s/file%d.txt' % (FOLDER, r))
-                self.assertEqual(f.readline(), 'this is file #%d.\n' % r)
-                f.close()
+                with sftp.open('%s/file%d.txt' % (FOLDER, r)) as f:
+                    self.assertEqual(f.readline(), 'this is file #%d.\n' % r)
                 numlist.remove(r)
         finally:
             for i in range(numfiles):
@@ -94,12 +88,11 @@ class BigSFTPTest (unittest.TestCase):
         kblob = (1024 * 'x')
         start = time.time()
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'w') as f:
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
@@ -107,11 +100,10 @@ class BigSFTPTest (unittest.TestCase):
             sys.stderr.write('%ds ' % round(end - start))
             
             start = time.time()
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-            for n in range(1024):
-                data = f.read(1024)
-                self.assertEqual(data, kblob)
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'r') as f:
+                for n in range(1024):
+                    data = f.read(1024)
+                    self.assertEqual(data, kblob)
 
             end = time.time()
             sys.stderr.write('%ds ' % round(end - start))
@@ -123,16 +115,15 @@ class BigSFTPTest (unittest.TestCase):
         write a 1MB file, with no linefeeds, using pipelining.
         """
         sftp = get_sftp()
-        kblob = ''.join([struct.pack('>H', n) for n in xrange(512)])
+        kblob = bytes().join([struct.pack('>H', n) for n in range(512)])
         start = time.time()
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'wb') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
@@ -140,22 +131,21 @@ class BigSFTPTest (unittest.TestCase):
             sys.stderr.write('%ds ' % round(end - start))
             
             start = time.time()
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-            f.prefetch()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'rb') as f:
+                f.prefetch()
 
-            # read on odd boundaries to make sure the bytes aren't getting scrambled
-            n = 0
-            k2blob = kblob + kblob
-            chunk = 629
-            size = 1024 * 1024
-            while n < size:
-                if n + chunk > size:
-                    chunk = size - n
-                data = f.read(chunk)
-                offset = n % 1024
-                self.assertEqual(data, k2blob[offset:offset + chunk])
-                n += chunk
-            f.close()
+                # read on odd boundaries to make sure the bytes aren't getting scrambled
+                n = 0
+                k2blob = kblob + kblob
+                chunk = 629
+                size = 1024 * 1024
+                while n < size:
+                    if n + chunk > size:
+                        chunk = size - n
+                    data = f.read(chunk)
+                    offset = n % 1024
+                    self.assertEqual(data, k2blob[offset:offset + chunk])
+                    n += chunk
 
             end = time.time()
             sys.stderr.write('%ds ' % round(end - start))
@@ -164,15 +154,14 @@ class BigSFTPTest (unittest.TestCase):
 
     def test_4_prefetch_seek(self):
         sftp = get_sftp()
-        kblob = ''.join([struct.pack('>H', n) for n in xrange(512)])
+        kblob = bytes().join([struct.pack('>H', n) for n in range(512)])
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'wb') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
             
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
@@ -180,21 +169,20 @@ class BigSFTPTest (unittest.TestCase):
             start = time.time()
             k2blob = kblob + kblob
             chunk = 793
-            for i in xrange(10):
-                f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-                f.prefetch()
-                base_offset = (512 * 1024) + 17 * random.randint(1000, 2000)
-                offsets = [base_offset + j * chunk for j in xrange(100)]
-                # randomly seek around and read them out
-                for j in xrange(100):
-                    offset = offsets[random.randint(0, len(offsets) - 1)]
-                    offsets.remove(offset)
-                    f.seek(offset)
-                    data = f.read(chunk)
-                    n_offset = offset % 1024
-                    self.assertEqual(data, k2blob[n_offset:n_offset + chunk])
-                    offset += chunk
-                f.close()
+            for i in range(10):
+                with sftp.open('%s/hongry.txt' % FOLDER, 'rb') as f:
+                    f.prefetch()
+                    base_offset = (512 * 1024) + 17 * random.randint(1000, 2000)
+                    offsets = [base_offset + j * chunk for j in range(100)]
+                    # randomly seek around and read them out
+                    for j in range(100):
+                        offset = offsets[random.randint(0, len(offsets) - 1)]
+                        offsets.remove(offset)
+                        f.seek(offset)
+                        data = f.read(chunk)
+                        n_offset = offset % 1024
+                        self.assertEqual(data, k2blob[n_offset:n_offset + chunk])
+                        offset += chunk
             end = time.time()
             sys.stderr.write('%ds ' % round(end - start))
         finally:
@@ -202,15 +190,14 @@ class BigSFTPTest (unittest.TestCase):
 
     def test_5_readv_seek(self):
         sftp = get_sftp()
-        kblob = ''.join([struct.pack('>H', n) for n in xrange(512)])
+        kblob = bytes().join([struct.pack('>H', n) for n in range(512)])
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'wb') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
@@ -218,22 +205,21 @@ class BigSFTPTest (unittest.TestCase):
             start = time.time()
             k2blob = kblob + kblob
             chunk = 793
-            for i in xrange(10):
-                f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-                base_offset = (512 * 1024) + 17 * random.randint(1000, 2000)
-                # make a bunch of offsets and put them in random order
-                offsets = [base_offset + j * chunk for j in xrange(100)]
-                readv_list = []
-                for j in xrange(100):
-                    o = offsets[random.randint(0, len(offsets) - 1)]
-                    offsets.remove(o)
-                    readv_list.append((o, chunk))
-                ret = f.readv(readv_list)
-                for i in xrange(len(readv_list)):
-                    offset = readv_list[i][0]
-                    n_offset = offset % 1024
-                    self.assertEqual(ret.next(), k2blob[n_offset:n_offset + chunk])
-                f.close()
+            for i in range(10):
+                with sftp.open('%s/hongry.txt' % FOLDER, 'rb') as f:
+                    base_offset = (512 * 1024) + 17 * random.randint(1000, 2000)
+                    # make a bunch of offsets and put them in random order
+                    offsets = [base_offset + j * chunk for j in range(100)]
+                    readv_list = []
+                    for j in range(100):
+                        o = offsets[random.randint(0, len(offsets) - 1)]
+                        offsets.remove(o)
+                        readv_list.append((o, chunk))
+                    ret = f.readv(readv_list)
+                    for i in range(len(readv_list)):
+                        offset = readv_list[i][0]
+                        n_offset = offset % 1024
+                        self.assertEqual(next(ret), k2blob[n_offset:n_offset + chunk])
             end = time.time()
             sys.stderr.write('%ds ' % round(end - start))
         finally:
@@ -247,28 +233,26 @@ class BigSFTPTest (unittest.TestCase):
         sftp = get_sftp()
         kblob = (1024 * 'x')
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'w') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
 
             for i in range(10):
-                f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
+                with sftp.open('%s/hongry.txt' % FOLDER, 'r') as f:
+                    f.prefetch()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'r') as f:
                 f.prefetch()
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-            f.prefetch()
-            for n in range(1024):
-                data = f.read(1024)
-                self.assertEqual(data, kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+                for n in range(1024):
+                    data = f.read(1024)
+                    self.assertEqual(data, kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
         finally:
             sftp.remove('%s/hongry.txt' % FOLDER)
@@ -278,35 +262,33 @@ class BigSFTPTest (unittest.TestCase):
         verify that prefetch and readv don't conflict with each other.
         """
         sftp = get_sftp()
-        kblob = ''.join([struct.pack('>H', n) for n in xrange(512)])
+        kblob = bytes().join([struct.pack('>H', n) for n in range(512)])
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'wb') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
             
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
 
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-            f.prefetch()
-            data = f.read(1024)
-            self.assertEqual(data, kblob)
-            
-            chunk_size = 793
-            base_offset = 512 * 1024
-            k2blob = kblob + kblob
-            chunks = [(base_offset + (chunk_size * i), chunk_size) for i in range(20)]
-            for data in f.readv(chunks):
-                offset = base_offset % 1024
-                self.assertEqual(chunk_size, len(data))
-                self.assertEqual(k2blob[offset:offset + chunk_size], data)
-                base_offset += chunk_size
+            with sftp.open('%s/hongry.txt' % FOLDER, 'rb') as f:
+                f.prefetch()
+                data = f.read(1024)
+                self.assertEqual(data, kblob)
+
+                chunk_size = 793
+                base_offset = 512 * 1024
+                k2blob = kblob + kblob
+                chunks = [(base_offset + (chunk_size * i), chunk_size) for i in range(20)]
+                for data in f.readv(chunks):
+                    offset = base_offset % 1024
+                    self.assertEqual(chunk_size, len(data))
+                    self.assertEqual(k2blob[offset:offset + chunk_size], data)
+                    base_offset += chunk_size
 
-            f.close()
             sys.stderr.write(' ')
         finally:
             sftp.remove('%s/hongry.txt' % FOLDER)
@@ -317,26 +299,24 @@ class BigSFTPTest (unittest.TestCase):
         returned as a single blob.
         """
         sftp = get_sftp()
-        kblob = ''.join([struct.pack('>H', n) for n in xrange(512)])
+        kblob = bytes().join([struct.pack('>H', n) for n in range(512)])
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w')
-            f.set_pipelined(True)
-            for n in range(1024):
-                f.write(kblob)
-                if n % 128 == 0:
-                    sys.stderr.write('.')
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'wb') as f:
+                f.set_pipelined(True)
+                for n in range(1024):
+                    f.write(kblob)
+                    if n % 128 == 0:
+                        sys.stderr.write('.')
             sys.stderr.write(' ')
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
             
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r')
-            data = list(f.readv([(23 * 1024, 128 * 1024)]))
-            self.assertEqual(1, len(data))
-            data = data[0]
-            self.assertEqual(128 * 1024, len(data))
+            with sftp.open('%s/hongry.txt' % FOLDER, 'rb') as f:
+                data = list(f.readv([(23 * 1024, 128 * 1024)]))
+                self.assertEqual(1, len(data))
+                data = data[0]
+                self.assertEqual(128 * 1024, len(data))
             
-            f.close()
             sys.stderr.write(' ')
         finally:
             sftp.remove('%s/hongry.txt' % FOLDER)
@@ -348,9 +328,8 @@ class BigSFTPTest (unittest.TestCase):
         sftp = get_sftp()
         mblob = (1024 * 1024 * 'x')
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w', 128 * 1024)
-            f.write(mblob)
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'w', 128 * 1024) as f:
+                f.write(mblob)
 
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
         finally:
@@ -365,21 +344,26 @@ class BigSFTPTest (unittest.TestCase):
         t.packetizer.REKEY_BYTES = 512 * 1024
         k32blob = (32 * 1024 * 'x')
         try:
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'w', 128 * 1024)
-            for i in xrange(32):
-                f.write(k32blob)
-            f.close()
-            
+            with sftp.open('%s/hongry.txt' % FOLDER, 'w', 128 * 1024) as f:
+                for i in range(32):
+                    f.write(k32blob)
+
             self.assertEqual(sftp.stat('%s/hongry.txt' % FOLDER).st_size, 1024 * 1024)
-            self.assertNotEquals(t.H, t.session_id)
+            self.assertNotEqual(t.H, t.session_id)
             
             # try to read it too.
-            f = sftp.open('%s/hongry.txt' % FOLDER, 'r', 128 * 1024)
-            f.prefetch()
-            total = 0
-            while total < 1024 * 1024:
-                total += len(f.read(32 * 1024))
-            f.close()
+            with sftp.open('%s/hongry.txt' % FOLDER, 'r', 128 * 1024) as f:
+                f.prefetch()
+                total = 0
+                while total < 1024 * 1024:
+                    total += len(f.read(32 * 1024))
         finally:
             sftp.remove('%s/hongry.txt' % FOLDER)
             t.packetizer.REKEY_BYTES = pow(2, 30)
+
+
+if __name__ == '__main__':
+    from tests.test_sftp import SFTPTest
+    SFTPTest.init_loopback()
+    from unittest import main
+    main()
